package com.rjconsultores.ventaboletos.web.utilerias.menu;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.zkoss.zk.ui.Desktop;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Menu;
import org.zkoss.zul.Menuitem;
import org.zkoss.zul.Menupopup;
import org.zkoss.zul.api.Menubar;

import com.rjconsultores.ventaboletos.entidad.PerfilFuncion;
import com.rjconsultores.ventaboletos.entidad.UsuarioPerfil;
import com.rjconsultores.ventaboletos.web.utilerias.menu.exception.MenuConfiguracionException;
import com.rjconsultores.ventaboletos.web.utilerias.menu.exception.MenuNoExisteException;

public class MenuFactoryPropertiesImpl implements MenuFactory {

	private List<No> tree = new ArrayList<No>();// Una lista con los itens del menu
	private BufferedReader menuReader; // El reader para leer el properties como un archivo texto
	private Properties miProperties; // El properties cargado normalmente. Asi se puede ocupar con las claves, etc
	private Desktop desktop; // Desktop de la aplicacin
	private List<UsuarioPerfil> listUsuarioPerfil;// El perfil del usuario
	private List<String> listClavesPermisos;

	public MenuFactoryPropertiesImpl(Desktop desktop, List<UsuarioPerfil> listUsuarioPerfil) {
		super();
		this.desktop = desktop;
		this.listUsuarioPerfil = listUsuarioPerfil;
		initListClavesPermissos();
	}

	private void initListClavesPermissos() {
		listClavesPermisos = new ArrayList<String>();
		for (UsuarioPerfil up : listUsuarioPerfil) {
			List<PerfilFuncion> listPerfilFuncion = up.getPerfil().getPerfilFuncionList();
			for (PerfilFuncion pf : listPerfilFuncion) {
				listClavesPermisos.add(pf.getFuncionSistema().getDescruta());
			}
		}
	}

	/**
	 * Empeza la construccion del menu
	 * 
	 * @param menuBar
	 */
	@Override
	public void construirMenu(Menubar menuBar) {
		try {
			this.cargarMenuProperties();
			this.contruir(null);

			for (No no : tree) {
				String klass = miProperties.getProperty(no.getNombre());

				Menu menu = this.createMenu(klass);
				menuBar.appendChild(menu);

				this.contruirMenus(no, menu);

			}
		} catch (IOException e) {
			new MenuConfiguracionException("No fue posible cargar el archivo de menu.", e);
		}
	}

	/**
	 * Hace la construccion de los menus
	 * 
	 * @param no
	 * @param menu
	 */
	private void contruirMenus(No no, Menu menu) {
		for (No noH : no.getHijos()) {
			String klass = miProperties.getProperty(noH.getNombre());
			if (noH.getHijos().isEmpty()) {
				Menupopup menupopup = menu.getMenupopup();
				if (menupopup == null) {
					menupopup = new Menupopup();
				}
				menupopup.appendChild(this.createMenuItem(klass));

				menu.appendChild(menupopup);
			} else {
				Menu menuHijo = this.createMenu(klass);

				Menupopup nuevoMenupopup = new Menupopup();
				nuevoMenupopup.appendChild(menuHijo);
				Menupopup menupopup = menu.getMenupopup();

				menupopup.appendChild(nuevoMenupopup);
				contruirMenus(noH, menuHijo);
			}
		}
	}

	/**
	 * Carga el archivo properties en una estructura de arbol
	 * 
	 * @param noPadre
	 * @throws IOException
	 */
	public void contruir(No noPadre) throws IOException {
		String linea = menuReader.readLine();

		if (linea == null) {
			return;
		}

		String[] arregloLinea = linea.split("=");

		if (noPadre == null) {
			No padre = new No(arregloLinea[0]);
			tree.add(padre);
			this.contruir(padre);
		} else {
			No hijo = new No(arregloLinea[0]);

			if (hijo.getNivel() == 1) {
				tree.add(hijo);
				contruir(hijo);
				return;
			}
			if ((noPadre.getNivel() + 1) == hijo.getNivel()) {
				noPadre.add(hijo);
				contruir(hijo);
			} else {
				for (No no : tree) {
					if (((no.getNivel() + 1) == hijo.getNivel()) && (hijo.getNombre().startsWith(no.getNombre()))) {
						no.add(hijo);
						contruir(no);
						break;

					}
				}
			}
		}
	}

	/**
	 * Crea un nuevo menu
	 * 
	 * @param strClass
	 * @return
	 */
	private Menu createMenu(String strClass) {
		ItemMenuSistema item = this.crearNueboItem(strClass);

		if (item != null) {
			Menu j = new Menu(item.getDescItemMenu());
//			if (item.getClaveMenu() != null) { no  usado. O arquivo de menu que deve controlar isso
//				j.setVisible(listClavesPermisos.contains(item.getClaveMenu()));
//			}
			return j;
		}
		return null;
	}

	/**
	 * Crea un nuevo menuItem
	 * 
	 * @param strClass
	 * @return
	 */
	private Menuitem createMenuItem(String strClass) {
		final ItemMenuSistema item = this.crearNueboItem(strClass);

		if (item != null) {
			Menuitem j = new Menuitem(item.getDescItemMenu());
			j.addEventListener("onClick", new EventListener() {
				@Override
				public void onEvent(Event arg0) throws Exception {
					item.ejecutar();
				}
			});
			if (item.getClaveMenu() != null) {
				j.setVisible(listClavesPermisos.contains(item.getClaveMenu()));
			}
			return j;
		}
		return null;
	}

	/**
	 * Crea una instancia del menu especificado
	 * 
	 * @param strClass
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	private ItemMenuSistema crearNueboItem(String strClass) {
		ItemMenuSistema classe = null;
		try {
			Class c = Class.forName(strClass);

			classe = (ItemMenuSistema) c.newInstance();
			classe.setDesktop(desktop);
			// Constructor constructor = c.getConstructor(new Class[]{Desktop.class});
			// classe = (ItemMenuSistema) constructor.newInstance(Executions.getCurrent().getDesktop());
		} catch (ClassNotFoundException e) {
			throw new MenuNoExisteException("El menu " + strClass + " no fue encontrado.", e);
		} catch (InstantiationException e) {
			throw new MenuNoExisteException("No se puede crear el menu " + strClass + ".", e);
		} catch (IllegalAccessException e) {
			throw new MenuNoExisteException("No fue posible accesar " + strClass + ".", e);
		}

		return classe;
	}

	/**
	 * Hace la carga del properties
	 * 
	 * @throws IOException
	 */
	private void cargarMenuProperties() throws IOException {
		InputStream miStream = this.getClass().getResourceAsStream("/com/rjconsultores/ventaboletos/web/cliente/menu/menu.properties");
		menuReader = new BufferedReader(new InputStreamReader(miStream));

		InputStream miStream2 = this.getClass().getResourceAsStream("/com/rjconsultores/ventaboletos/web/cliente/menu/menu.properties");

		miProperties = new Properties();
		miProperties.load(miStream2);
	}

	/**
	 * Clase con la estructura del menu leda del archivo properties
	 * 
	 * @author gleimar
	 * 
	 */
	public class No {
		String nombre;
		List<No> hijos;

		public No(String nombre) {
			this.nombre = nombre;
			this.hijos = new ArrayList<No>();
		}

		public No(String nombre, String hijo) {
			super();
			this.nombre = nombre;
			hijos = new ArrayList<No>();
			hijos.add(new No(hijo));
		}

		public No(String nombre, List<No> hijos) {
			super();
			this.nombre = nombre;
			this.hijos = hijos;
		}

		public String getNombre() {
			return nombre;
		}

		public void setNombre(String nombre) {
			this.nombre = nombre;
		}

		public List<No> getHijos() {
			return hijos;
		}

		public void setHijos(List<No> hijos) {
			this.hijos = hijos;
		}

		public boolean add(No e) {
			return hijos.add(e);
		}

		public int getNivel() {
			return nombre.split("\\.").length;
		}

	}

}
